/*
	Filename:		gregorian.c
	Date Created:		21/05/1995
	Date Last Edited:	14/08/1995
	Author:			Charles O. Gamble
	Purpose:		Gregorian date routines source code

	History:

	21/05/1995 -> Created file.
	06/08/1995 -> Separated this file from reminder.c
	14/08/1995 -> Changed some constants to Long (to aid porting).
*/


#include	"gregorian.h"




/*
	Name:		gregorian
	Purpose:	Calculates the Gregorian number for a given date
	Parameters:	day (1-31), month (1-12), year
	Returns:	Gregorian date value for the given date
	Status:		Finished
	Bugs: 		None known
*/
unsigned long gregorian(int d, int m, int y)
{
	unsigned long g;

	/* Now work out the Gregorian date */
	if ((m == JAN) || (m == FEB))
	{
		g = (365L*y)+(31L*(m-1L))+d+((y-1L)/4L)-(3L*((y+99L)/100L))/4L;
	}
	else
	{
		g = (365L*y)+(31L*(m-1L))+d+(y/4L)-(((4L*m)+23L)/10L)-(3L*((y/100L)+1L))/4L;
	}

	return (g);
}




/*
	Name:		between
	Purpose:	Calculates the absolute number of days between two dates
	Parameters:	day1 (1-31), month1 (1-12), year1, day2 (1-31), month2 (1-12), year2
	Returns:	Number of days (unsigned long)
	Status:		Finished
	Bugs: 		None known
*/
unsigned long between(int d1, int m1, int y1, int d2, int m2, int y2)
{
	unsigned long g1, g2;

	g1 = gregorian(d1, m1, y1);
	g2 = gregorian(d2, m2, y2);

	if (g1 > g2)
		return (g1 - g2);
	else
		return (g2 - g1);
}




/*
	Name:		dow
	Purpose:	Calculates the day of the week
	Parameters:	day (1-31), month (1-12), year
	Returns:	Number representing day of the week (int)
	Status:		Finished
	Bugs: 		None known
*/
int dow(int d, int m, int y)
{
	unsigned long g;
	int day;

	g = gregorian(d, m, y);

	day = (int) (g % 7);

	return (day);
}




/*
	Name:		leap
	Purpose:	Determines whether a given year is a leap year or not
	Parameters:	Year (int)
	Returns:	TRUE if y is a leap year, FALSE otherwise
	Status:		Finished
	Bugs: 		None known
*/
int leap(int y)
{
	int criterion1, criterion2;

	if ((y % 400) == 0)
		criterion1 = TRUE;
	else
		criterion1 = FALSE;

	if (((y % 4) == 0) && ((y % 100) != 0))
		criterion2 = TRUE;
	else
		criterion2 = FALSE;

	if (!criterion1 && !criterion2)
		return (FALSE);

	if (criterion1 && criterion2)
		return (FALSE);

	return (TRUE);
}




/*
	Name:		days_in_month
	Purpose:	Calculates the number of days in a month
	Parameters:	Month (1-12), year
	Returns:	The number of days in a given month (int)
	Status:		Finished
	Bugs: 		None known
*/
int days_in_month(int m, int y)
{
	int leapyr;

	leapyr = leap(y);

	if ((m==4) || (m==6) || (m==9) || (m==11))
		return (30);

	if ((m==2) && (!leapyr))
		return (28);

	if ((m==2) && (leapyr))
		return (29);

	return (31);
}




/*
	Name:		legitimate
	Purpose:	Determines whether a date is valid
	Parameters:	day (1-31), month (1-12), year
	Returns:	TRUE if date is legitimate, FALSE otherwise
	Status:		Finished
	Bugs: 		None known
*/
int legitimate(int d, int m, int y)
{
	if ((m < 1) || (m > 12))
		return (FALSE);

	if (gregorian(d, m, y) < 578101L)
		return (FALSE);

	if ((d < 1) || (d > days_in_month(m, y)))
		return (FALSE);

	return (TRUE);
}




/* End of file - gregorian.c *************************************************/



