/*
	Filename:		reminder.c
	Date Created:		21/05/1995
	Date Last Edited:	13/06/1996
	Author:			Charles O. Gamble
	Purpose:		Reminder source code

	History:

	21/05/1995 -> Created file.
	29/05/1995 -> Changed messages, added check for MAX_DATES.
	30/05/1995 -> Added 'legitimate' checks in CheckDate().
	06/08/1995 -> Separated date routines into separate file, tidied up.
	09/08/1995 -> Created separate header file, added --help, --verbose and --version.
	13/08/1995 -> Added -days, -ukdate and -usdate flags.
		      Added PrintReminder routines.
	14/08/1995 -> Fixed CheckDate() to use dow(d, m, nextyear) on year boundaries.
		      Removed unused integer 'i' from CheckDate().
	05/05/1996 -> Removed LogMsg calls, replaced with fprintf's.
	26/05/1996 -> Changed command-line parsing to allow multiple files.
		      Altered comment-handling so that the # does not have to be in the first column.
		      Added SkipLeadingWhitespace().
		      Altered input file format so that the order is now MONTH,DAY instead of DAY,MONTH.
		      Added invalid-date warning in CheckDate().
	12/06/1996 -> Moved #include's to reminder.h.
	13/06/1996 -> Now prints author's name under program title.
*/


#include	"reminder.h"
#include	"gregorian.h"




/* Globals *******************************************************************/

char	gsz_Version[] = "1.02";		/* Version of the program */
int	day, month, year;		/* Today's date */
char	gsz_Day[7][10];			/* Days of the week */
int	gb_Verbose = FALSE;		/* Lots of messages */
int	gb_USDate = FALSE;		/* Default to UK date order */
int	gi_NumberDaysWarning = 10;	/* Default number of days warning */

/* End Globals ***************************************************************/




/*
	Name:		main
	Purpose:	Main routine for Reminder
	Parameters:	Number of command-line parameters, pointer to parameters
	Returns:	0 if execution successful
	Status:		Finished
	Bugs: 		None known
*/
int main(int argc, char *argv[])
{
	int arg, days;

	if (argc < 2)
	{
		fprintf(stdout, "\nreminder v%s (built %s, %s)\n", gsz_Version, __TIME__, __DATE__);
		fprintf(stdout, "Charles O. Gamble\n\n");
		Usage();		/* Display help msg */
		exit(1);		/* Exit if no arguments supplied */
	}
		
	for (arg = 1; arg < argc; arg++)
	{
		/* Test for the HELP option */
		if (strcmp(argv[arg], "--help") == 0)
		{
			fprintf(stdout, "\nreminder v%s (built %s, %s)\n", gsz_Version, __TIME__, __DATE__);
			fprintf(stdout, "Charles O. Gamble\n\n");
			Usage();		/* Display help msg */
			exit(1);
		}

		/* Test for the VERSION option */
		if (strcmp(argv[arg], "--version") == 0)
		{
			fprintf(stdout, "\nreminder v%s (built %s, %s)\n", gsz_Version, __TIME__, __DATE__);
			fprintf(stdout, "Charles O. Gamble\n\n");
			exit(1);
		}
		
		/* Test for the VERBOSE option */
		if (strcmp(argv[arg], "--verbose") == 0)
		{
			strcpy(argv[arg], "");	/* Blank out argument once it has been used */
			gb_Verbose = TRUE;	/* Turn on messaging */
		}

		/* Test for the USDATE option */
		if (strcmp(argv[arg], "-usdate") == 0)
		{
			strcpy(argv[arg], "");	/* Blank out argument once it has been used */
			gb_USDate = TRUE;	/* Turn on US date order (mm/dd/yy) */
		}

		/* Test for the UKDATE option */
		if (strcmp(argv[arg], "-ukdate") == 0)
		{
			strcpy(argv[arg], "");	/* Blank out argument once it has been used */
			gb_USDate = FALSE;	/* Turn on UK date order (dd/mm/yy) */
		}

		/* Test for the DAYS option - PUT THIS TEST LAST */
		if (strncmp(argv[arg], "-days", 5) == 0)
		{
			if (sscanf(argv[arg], "-days%d", &days) == 1)
			{
				strcpy(argv[arg], "");	/* Blank out argument once it has been used */
				/* Number was given like '-days20' */
				if ((days >= 1) && (days <= 365))
					gi_NumberDaysWarning = days;
				else
					fprintf(stderr, "reminder: Number of days warning must be between 1 and 365 (inclusive)\n");
			}
			else
			{
				/* Number might have been given like '-days 20' */
				if (strlen(argv[arg]) == 5)
				{
					strcpy(argv[arg], "");	/* Blank out argument once it has been used */

					if (sscanf(argv[arg+1], "%d", &days) == 1)
					{
						strcpy(argv[arg+1], "");	/* Blank out argument once it has been used */
					
						if ((days >= 1) && (days <= 365))
						{
							gi_NumberDaysWarning = days;
						}
						else
							fprintf(stderr, "reminder: Number of days warning must be between 1 and 365 (inclusive)\n");

						arg++;				/* Skip over argument */
					}
					else
						fprintf(stderr, "reminder: Cannot get number of days\n");
				}
			}
		}
	}

	Initialise();			/* Set up arrays, etc. */
	for (arg = 1; arg < argc; arg++)
	{
		if (strlen(argv[arg]) > 0)
			ReadFile(argv[arg]);	/* Read input file */
	}

	if (gb_Verbose)
		fprintf(stdout, "Terminated OK\n");

	return (0);			/* Success */
}




/*
	Name:		Usage
	Purpose:	Displays usage message
	Parameters:	None
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void Usage()
{
	fprintf(stdout, "Usage:\n\n");
	fprintf(stdout, "\treminder [options] filename...\n\n");
	fprintf(stdout, "Where 'options' can be made up of:\n");
	fprintf(stdout, "-ukdate\n-usdate\n-days n\n--help\n--version\n--verbose\n\n");
}




/*
	Name:		Initialise
	Purpose:	Sets up any globals, etc.
	Parameters:	None
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void Initialise()
{
	struct timeb	today_t;
	struct tm	*today;

	strcpy(gsz_Day[SAT], "Saturday");
	strcpy(gsz_Day[SUN], "Sunday");
	strcpy(gsz_Day[MON], "Monday");
	strcpy(gsz_Day[TUE], "Tuesday");
	strcpy(gsz_Day[WED], "Wednesday");
	strcpy(gsz_Day[THU], "Thursday");
	strcpy(gsz_Day[FRI], "Friday");

	ftime(&today_t);
	today = localtime(&(today_t.time));

	day = today->tm_mday;			/* Get today's details */
	month = today->tm_mon + 1;
	year = today->tm_year + 1900;

	if (gb_Verbose)
	{
		fprintf(stdout, "\nreminder v%s (built %s, %s)\n\n", gsz_Version, __TIME__, __DATE__);
		fprintf(stdout, "Initialised OK\n");
		fprintf(stdout, "Number of days warning: %d\n", gi_NumberDaysWarning);
		if (gb_USDate)
			fprintf(stdout, "Using US date format (month/day/year)\n");
		else
			fprintf(stdout, "Using British date format (day/month/year)\n");
	}
}




/*
	Name:		SkipLeadingWhitespace
	Purpose:	Finds first non-whitespace character in a string
	Parameters:	String to search
	Returns:	Pointer to first non-whitespace character in string
	Status:		Finished
	Bugs: 		None known
*/
char *SkipLeadingWhitespace(char *string)
{
	int	i;
	
	if (!string)
		return (NULL);		/* We have a NULL pointer */
	else
	{
		/* Loop through string looking for the first non-whitespace character */
		for (i = 0; i < strlen(string); i++)
		{
			if (!isspace(string[i]))
				return (&(string[i]));	/* Found it, return address */
		}
	}

	return (NULL);			/* No non-whitespace characters found in string */
}




/*
	Name:		ReadFile
	Purpose:	Reads in the input file
	Parameters:	Filename of input file
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void ReadFile(char *filename)
{
	FILE	*fptr;
	char	line[MAX_LINE_LENGTH+1];
	int 	d, m;
	char	*description, *nl, *first_char;
	Tdate	date;

	if ((fptr = fopen(filename, "r")) == NULL)
	{
		fprintf(stderr, "reminder: Error whilst opening \'%s\'\n", filename);
		perror("");
		return;
	}

	if (gb_Verbose)
		fprintf(stdout, "\'%s\' opened OK\n", filename);

	while (!feof(fptr))
	{
		do
		{
			fgets(line, MAX_LINE_LENGTH, fptr);	/* Read a line */
			first_char = SkipLeadingWhitespace(line);	/* Find first non-whitespace character */
		} while(((!first_char) || (*first_char == '#') || (strlen(line) <= 1)) && (!feof(fptr)));

		if (!feof(fptr))
		{
			if (sscanf(line, "%d,%d", &m, &d) == 2)
			{
				date.d = d;
				date.m = m;
				if ((description = strchr(line, '$')) == NULL)
					strcpy(date.desc, "Mystery");
				else
				{
					description++;	/* Skip past $ character */
					if ((nl = strrchr(description, '\n')) != NULL)
						*nl = '\0';		/* Strip off newline */
						strcpy(date.desc, description);
				}
				CheckDate(&date);	/* Check this date */
			}
		}
	}
	fclose(fptr);

	if (gb_Verbose)
		fprintf(stdout, "\'%s\' closed OK\n", filename);
}




/*
	Name:		CheckDate
	Purpose:	Checks the date given and prints msg if necessary
	Parameters:	Pointer to date structure
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void CheckDate(Tdate *date)
{
	int	d, m, nextyear;

	nextyear = year + 1;

	d = date->d;
	m = date->m;

	if (gb_Verbose)
	{
		if (gb_USDate)
			fprintf(stdout, "Checking date: %d/%d\n", m, d);
		else
			fprintf(stdout, "Checking date: %d/%d\n", d, m);
			
		/* If date is not valid for a leap year - it is not valid at all (1996 was a leap year) */
		if (!legitimate(d, m, 1996))
		{
			if (gb_USDate)
				fprintf(stdout, "Invalid date: %d/%d\n", m, d);
			else
				fprintf(stdout, "Invalid date: %d/%d\n", d, m);
			return;
		}
	}

	if (gregorian(d, m, year) >= gregorian(day, month, year))
	{
		if ((gregorian(d, m, year) == gregorian(day, month, year)) && (legitimate(d, m, year)))
			PrintTodayReminder(date->desc, dow(d, m, year), d, m, year);
		else
		{
			if ((between(d, m, year, day, month, year) == 1) && (legitimate(d, m, year)))
				PrintTomorrowReminder(date->desc, dow(d, m, year), d, m, year);
			else
			{
				if ((between(d, m, year, day, month, year) <= gi_NumberDaysWarning) && (legitimate(d, m, year)))
					PrintReminder(date->desc, dow(d, m, year), d, m, year, between(d, m, year, day, month, year));
			}
		}
	}
	else
	{
		/* Handles end/beginning year boundary  - NOT REALLY TESTED YET */
		if ((gregorian(d, m, nextyear) > gregorian(day, month, year)) && (legitimate(d, m, nextyear)))
		{
			if ((between(d, m, nextyear, day, month, year) == 1) && (legitimate(d, m, nextyear)))
				PrintTomorrowReminder(date->desc, dow(d, m, nextyear), d, m, nextyear);
			else
			{
				if ((between(d, m, nextyear, day, month, year) <= gi_NumberDaysWarning) && (legitimate(d, m, nextyear)))
					PrintReminder(date->desc, dow(d, m, nextyear), d, m, nextyear, between(d, m, nextyear, day, month, year));
			}
		}
	}
}




/*
	Name:		PrintTodayReminder
	Purpose:	Print reminder msg for today
	Parameters:	Msg, dow, day, month, year
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void PrintTodayReminder(char *msg, int day_index, int d, int m, int y)
{
	if (gb_USDate)
		printf("REMINDER: %s today, %s %d/%d/%d\n", msg, gsz_Day[day_index], m, d, y);
	else
		printf("REMINDER: %s today, %s %d/%d/%d\n", msg, gsz_Day[day_index], d, m, y);
}




/*
	Name:		PrintTomorrowReminder
	Purpose:	Print reminder msg for tomorrow
	Parameters:	Msg, dow, day, month, year
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void PrintTomorrowReminder(char *msg, int day_index, int d, int m, int y)
{
	if (gb_USDate)
		printf("REMINDER: %s tomorrow, %s %d/%d/%d\n", msg, gsz_Day[day_index], m, d, y);
	else
		printf("REMINDER: %s tomorrow, %s %d/%d/%d\n", msg, gsz_Day[day_index], d, m, y);
}




/*
	Name:		PrintReminder
	Purpose:	Print reminder msg
	Parameters:	Msg, dow, day, month, year, difference between dates
	Returns:	Nothing
	Status:		Finished
	Bugs: 		None known
*/
void PrintReminder(char *msg, int day_index, int d, int m, int y, unsigned long difference)
{
	if (gb_USDate)
		printf("REMINDER: %s on %s %d/%d/%d (%ld days time)\n", msg, gsz_Day[day_index], m, d, y, difference);
	else
		printf("REMINDER: %s on %s %d/%d/%d (%ld days time)\n", msg, gsz_Day[day_index], d, m, y, difference);
}




/* End Of File - reminder.c **************************************************/
